/**
 * @Author：杨振忠
 * @Date 21/06/22
 * @Version 1.0
 * @Description 工具类
 */


/**
 * 时间格式化
 * @param data <date> 格式化时间
 * @param fmt <string> 格式化类型 例如："yyyy-MM-dd"
 * @returns {*|void|string}
 */
export function formatDate(data, fmt) {
	if (/(y+)/.test(fmt)) {
		fmt = fmt.replace(
			RegExp.$1,
			(data.getFullYear() + "").substr(4 - RegExp.$1.length)
		);
	}

	let o = {
		"M+": data.getMonth() + 1,
		"d+": data.getDate(),
		"h+": data.getHours(),
		"m+": data.getMinutes(),
		"s+": data.getSeconds()
	};

	for (let k in o) {
		if (new RegExp(`(${k})`).test(fmt)) {
			let str = o[k] + "";
			fmt = fmt.replace(
				RegExp.$1,
				RegExp.$1.length === 1 ? str : padLeftZero(str)
			);
		}
	}

	return fmt;
}

/**
 * 保留两位
 * @param str <string | number> 将要处理字符
 * @returns {string}
 */
function padLeftZero(str) {
	return ("00" + str).substr(str.length);
}

/**
 * @desc 函数防抖
 * @param func 函数
 * @param wait 延迟执行毫秒数
 * @param immediate true 表立即执行，false 表非立即执行
 */
export function debounce(func, wait, immediate) {
	let timeout;

	return function() {
		let context = this;
		let args = arguments;

		if (timeout) clearTimeout(timeout);
		if (immediate) {
			var callNow = !timeout;
			timeout = setTimeout(() => {
				timeout = null;
			}, wait)
			if (callNow) func.apply(context, args)
		} else {
			timeout = setTimeout(function() {
				func.apply(context, args)
			}, wait);
		}
	}
}


/**
 * @desc 函数节流
 * @param func 函数
 * @param wait 延迟执行毫秒数
 * @param type 1 表时间戳版，2 表定时器版
 */
export function throttle(func, wait, type) {
	if (type === 1) {
		let previous = 0;
	} else if (type === 2) {
		let timeout;
	}
	return function() {
		let context = this;
		let args = arguments;
		if (type === 1) {
			let now = Date.now();

			if (now - previous > wait) {
				func.apply(context, args);
				previous = now;
			}
		} else if (type === 2) {
			if (!timeout) {
				timeout = setTimeout(() => {
					timeout = null;
					func.apply(context, args)
				}, wait)
			}
		}
	}
}

/**
 * @desc 是否包含成员
 * @param obj 目标对象
 * @param key 成员名
 * @returns Boolean
 */
export function hasOwn(obj, key) {
	return obj.hasOwnProperty(key);
}

/**
 * @desc 深拷贝(支持复杂对象)
 * @param obj 目标对象
 * @returns {*}
 */
export function deepCopy(obj) {
	let newObj;
	switch (typeof obj) {
		case 'object':
			newObj = new Object;
			break;
		case 'array':
			newObj = new Array;
			break;
		default:
			return obj;
			break;
	}
	for (let key in obj) {
		if (typeof obj[key] == 'object' || typeof obj[key] == 'array') {
			newObj[key] = deepCopy(obj[key]);
		} else {
			newObj[key] = obj[key];
		}
	}
	return newObj;
}